<?php
/**
 * FuzeWorks Async Library
 *
 * The FuzeWorks PHP FrameWork
 *
 * Copyright (C) 2013-2020 TechFuze
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * @author    TechFuze
 * @copyright Copyright (c) 2013 - 2020, TechFuze. (http://techfuze.net)
 * @license   https://opensource.org/licenses/MIT MIT License
 *
 * @link  http://techfuze.net/fuzeworks
 * @since Version 1.0.0
 *
 * @version Version 1.0.0
 */

namespace FuzeWorks\Async;

/**
 * Interface Executor
 *
 *
 * @todo Implement ListRunningTasks
 * @package FuzeWorks\Async
 */
interface Executor
{

    /**
     * Executor constructor.
     *
     * Parameters are a unique array which can differ for each Executor.
     *
     * @param array $parameters
     */
    public function __construct(array $parameters);

    // Control methods
    /**
     * Start executing a task.
     *
     * Returns the Task since it makes modifications. Has to be modified in TaskStorage by SuperVisor.
     *
     * @param Task $task
     * @param bool $post
     * @return Task
     */
    public function startTask(Task $task, bool $post = false): Task;

    /**
     * Stops the execution of a task.
     *
     * Returns the task since it makes modifications. Has to be modified in TaskStorage by SuperVisor.
     *
     * @param Task $task
     * @param bool $harshly True to KILL a process
     * @return Task|null    Returns modified Task on success, or null if no PID is found
     */
    public function stopTask(Task $task, bool $harshly = false): ?Task;

    // Task info
    public function getTaskRunning(Task $task): bool;
    public function getTaskStats(Task $task): ?array;
}