<?php
/**
 * FuzeWorks Async Library
 *
 * The FuzeWorks PHP FrameWork
 *
 * Copyright (C) 2013-2020 TechFuze
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * @author    TechFuze
 * @copyright Copyright (c) 2013 - 2020, TechFuze. (http://techfuze.net)
 * @license   https://opensource.org/licenses/MIT MIT License
 *
 * @link  http://techfuze.net/fuzeworks
 * @since Version 1.0.0
 *
 * @version Version 1.0.0
 */

namespace FuzeWorks\Async;
interface Handler
{

    /**
     * Gets invoked upon being added to the Task
     *
     * @param Task $task
     * @return mixed
     * @throws TasksException
     */
    public function init(Task $task);

    /**
     * Retrieve the parent handler that will first handle this task, before this child Handler
     *
     * @return Handler|null
     */
    public function getParentHandler(): ?Handler;

    /**
     * Set the parent handler that will fire before this Handler
     *
     * @param Handler $parentHandler
     */
    public function setParentHandler(Handler $parentHandler): void;

    /**
     * Import the parent output into the child
     *
     * @param string $input
     */
    public function setParentInput(string $input): void;

    /**
     * The handler method used to handle this task.
     * This handler will execute the actual task.
     *
     * Should return true when successful. Return false when failed.
     *
     * @param Task $task
     * @return bool
     */
    public function primaryHandler(Task $task): bool;

    /**
     * Any output generated by primaryHandler should be returned here.
     *
     * @return string
     */
    public function getOutput(): string;

    /**
     * The handler method used after the primaryHandler if so requested
     *
     * The task will instruct the supervisor whether the postHandler is to be called.
     *
     * Remember: postHandler is called in a new process. Data is not shared between processes unless done manually.
     * @param Task $task
     */
    public function postHandler(Task $task);

    /**
     * Any output generated by postHandler should be returned here
     *
     * @return string
     */
    public function getPostOutput(): string;

}